import 'package:flutter/material.dart';

/// Tipos de animaciones
enum AnimationType { normal, fadeIn }

/// Parametros necesarios
class AnimateRouteTransition {
  final BuildContext context;
  final Widget child;
  final AnimationType animation;
  final Duration duration;
  final bool replacement;
  final Curve animateCurve;

  /// Inicializacion de variables.
  /// [context] es el context de la aplicación
  /// [child] es el widget de la pagina
  /// [replacement] es una opcion para que la pagina sea reemplazada
  AnimateRouteTransition(
      {@required this.context,
      @required this.child,
      this.animation = AnimationType.fadeIn,
      this.duration = const Duration(milliseconds: 300),
      this.replacement = false,
      this.animateCurve = Curves.easeInOut}) {
    switch (this.animation) {
      case AnimationType.normal:

        /// Animacion normal
        this._normalTransition();
        break;
      case AnimationType.fadeIn:

        /// Animacion FadeIn
        this._fadeInTransition();
        break;
    }
  }

  /// Push normal de la pagina.
  void _pushPage(Route route) => Navigator.push(context, route);

  /// Push Replacement normal de la pagina.
  void _pushReplacement(Route route) =>
      Navigator.pushReplacement(context, route);

  /// Codigo de tensicion normal.
  void _normalTransition() {
    final route = MaterialPageRoute(builder: (_) => this.child);
    (this.replacement) ? _pushReplacement(route) : _pushPage(route);
  }

  /// Codigo de tensicion fadein con PageRouterBuilder.
  void _fadeInTransition() {
    final route = PageRouteBuilder(
      pageBuilder: (_, __, ___) => this.child,
      transitionDuration: this.duration,
      transitionsBuilder: (_, animation, __, child) => FadeTransition(
          child: child,
          opacity: Tween<double>(begin: 0.0, end: 1.0).animate(
              CurvedAnimation(parent: animation, curve: this.animateCurve))),
    );

    (this.replacement) ? _pushReplacement(route) : _pushPage(route);
  }
}
